@@ -98,7 +98,7 @@ static char* get_ilistfile_path(const struct pbuilderconfig* pc)
   if (0>asprintf(&ilistfile, "%s/.ilist", pc->buildplace))
     {
       /* outofmemory */
-      fprintf(stderr, "cowdancer: out of memory.\n");
+      log_printf(log_error, "cowdancer: out of memory.");
       return NULL;
     }
   return ilistfile;
@@ -136,7 +136,7 @@ int break_cowlink(const char* s)
 	  /* child process, run cp */
 	  putenv("COWDANCER_IGNORE=yes");
 	  execl("/bin/cp", "/bin/cp", "-a", s, backup_file, NULL);
-	  perror("execl:cp:");
+	  log_perror("execl:cp:");
 	  exit(EXIT_FAILURE);
 	case -1:
 	  /* error condition in fork(); something is really wrong */
@@ -146,8 +146,8 @@ int break_cowlink(const char* s)
 	  /* parent process, waiting for cp -a to terminate */
 	  if(-1==waitpid(pid, &status, 0))
 	    {
-	      perror("waitpid:cp");
-	      fprintf(stderr, "%s: unexpected waitpid error when waiting for process %i with status %x\n",
+	      log_perror("waitpid:cp");
+	      log_printf(log_error, "%s: unexpected waitpid error when waiting for process %i with status %x",
 		      ilist_PRGNAME, pid, status);
 	      goto error_spm;
 	    }
@@ -155,21 +155,21 @@ int break_cowlink(const char* s)
 	  if (!WIFEXITED(status))
 	    {
 	      /* something unexpected */
-	      fprintf(stderr, "%s: unexpected WIFEXITED status in waitpid: %x\n", ilist_PRGNAME,
+	      log_printf(log_error, "%s: unexpected WIFEXITED status in waitpid: %x", ilist_PRGNAME,
 		      (int)status);
 	      goto error_spm;
 	    }
 	  else if (WEXITSTATUS(status))
 	    {
 	      /* cp -a failed */
-	      fprintf(stderr, "%s: cp -a failed for %s\n", ilist_PRGNAME, backup_file);
+	      log_printf(log_error, "%s: cp -a failed for %s", ilist_PRGNAME, backup_file);
 	      goto error_spm;
 	    }
 	  /* when cp -a succeeded, overwrite the target file from the temporary file with rename */
 	  else if (-1==rename(backup_file, s))
 	    {
-	      perror ("file overwrite with rename");
-	      fprintf(stderr, "while trying rename of %s to %s\n",  backup_file, s);
+	      log_perror("file overwrite with rename");
+	      log_printf(log_error, "while trying rename of %s to %s",  backup_file, s);
 	      goto error_spm;
 	    }
 	}
@@ -202,7 +202,7 @@ static int cpbuilder_internal_cowcopy(const struct pbuilderconfig* pc)
 {
   char *ilistfile;
 
-  printf(" -> Copying COW directory\n");
+  log_printf(log_info, "Copying COW directory");
   if (0!=rmrf_check_mount(pc->buildplace))
     return 1;
   if (0!=forkexeclp("cp", "cp", "-al", pc->basepath, pc->buildplace, NULL))
@@ -213,7 +213,7 @@ static int cpbuilder_internal_cowcopy(const struct pbuilderconfig* pc)
   if(!(ilistfile=get_ilistfile_path(pc)))
     {
       /* outofmemory */
-      fprintf(stderr, "cowdancer: out of memory.\n");
+      log_printf(log_error, "cowdancer: out of memory.");
       return 1;
     }
   if (unlink(ilistfile))
@@ -221,11 +221,11 @@ static int cpbuilder_internal_cowcopy(const struct pbuilderconfig* pc)
       /* if there was no ilist file in the beginning, that's not a
 	 problem.
        */
-      printf("I: unlink for ilistfile %s failed, it didn't exist?\n", ilistfile);
+      log_printf(log_info, "unlink for ilistfile %s failed, it didn't exist?", ilistfile);
     }
   else
     {
-      printf("I: removed stale ilistfile %s\n", ilistfile);
+      log_printf(log_info, "removed stale ilistfile %s", ilistfile);
     }
   free(ilistfile);
 
@@ -243,7 +243,7 @@ static int cpbuilder_internal_cowcopy(const struct pbuilderconfig* pc)
  */
 static int cpbuilder_internal_cleancow(const struct pbuilderconfig* pc)
 {
-  printf(" -> Cleaning COW directory\n");
+  log_printf(log_info, "Cleaning COW directory");
 
   if (!pc->debug && 0!=rmrf_check_mount(pc->buildplace))
     return 1;
@@ -263,13 +263,13 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile_)
 
   if (!dscfile)
     {
-      fprintf(stderr, "File not found: %s\n", dscfile_);
+      log_printf(log_error, "File not found: %s", dscfile_);
       return 1;
     }
 
   if (cpbuilder_internal_cowcopy(pc))
     {
-      fprintf(stderr, "Failed cowcopy.\n");
+      log_printf(log_error, "Failed cowcopy.");
       return 1;
     }
 
@@ -277,14 +277,14 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile_)
   if (0>asprintf(&buf_chroot, "chroot %s cow-shell", pc->buildplace))
     {
       /* outofmemory */
-      fprintf(stderr, "cowdancer: out of memory.\n");
+      log_printf(log_error, "cowdancer: out of memory.");
       return -1;
     }
 
   if(!(ilistfile=get_ilistfile_path(pc)))
     {
       /* outofmemory */
-      fprintf(stderr, "cowdancer: out of memory.\n");
+      log_printf(log_error, "cowdancer: out of memory.");
       return 1;
     }
 
@@ -304,7 +304,7 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile_)
       /* but first, try etch-workaround */
       if (pc->debian_etch_workaround)
 	{
-	  fprintf(stderr, "W: Trying my backup method for working with Debian Etch chroots.\n");
+	  log_printf(log_warn, "Trying my backup method for working with Debian Etch chroots.");
 	  if (forkexeclp("chroot",
 			 "chroot",
 			 pc->buildplace,
@@ -313,7 +313,7 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile_)
 			 NULL))
 	    {
 	      /* I failed, what can I do? noooo */
-	      fprintf(stderr, "E: failed running Debian Etch compatibility backup plan, I'm in panic, eek.\n");
+	      log_printf(log_error, "failed running Debian Etch compatibility backup plan, I'm in panic, eek.");
 	      return 1;
 	    }
 	}
@@ -324,7 +324,7 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile_)
 
 	     It probably doesn't work but try it anyway.
 	   */
-	  fprintf(stderr, "W: cowdancer-ilistcreate failed to run within chroot, falling back to old method\n");
+	  log_printf(log_warn, "cowdancer-ilistcreate failed to run within chroot, falling back to old method");
 	  ilistcreate(ilistfile,
 		      "find . -xdev -path ./home -prune -o \\( \\( -type l -o -type f \\) -a -links +1 -print0 \\) | xargs -0 stat --format '%d %i '");
 	}
@@ -332,7 +332,7 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile_)
   chdir(prevdir);
   free(prevdir);
 
-  printf(" -> Invoking pbuilder\n");
+  log_printf(log_info, "Invoking pbuilder");
   pbuildercommandline[1]="build";
   PBUILDER_ADD_PARAM("--buildplace");
   PBUILDER_ADD_PARAM(pc->buildplace);
@@ -364,7 +364,7 @@ int cpbuilder_build(const struct pbuilderconfig* pc, const char* dscfile_)
   if (ret < 128)
     cpbuilder_internal_cleancow(pc);
   else
-    printf("pbuilder build aborted, not cleaning\n");
+    log_printf(log_warn, "pbuilder build aborted, not cleaning");
 
   free(ilistfile);
   return ret;
@@ -375,7 +375,7 @@ int cpbuilder_create(const struct pbuilderconfig* pc)
   int ret;
 
   mkdir(pc->basepath,0777);
-  printf(" -> Invoking pbuilder\n");
+  log_printf(log_info, "Invoking pbuilder");
 
   pbuildercommandline[1]="create";
   PBUILDER_ADD_PARAM("--buildplace");
@@ -399,7 +399,7 @@ int cpbuilder_create(const struct pbuilderconfig* pc)
       char *extrapackages;
       if (0>asprintf(&extrapackages, "%s cowdancer", pc->extrapackages))
 	{
-	  fprintf(stderr, "cowdancer: out of memory.\n");
+	  log_printf(log_error, "cowdancer: out of memory.");
 	  ret = 1;
 	  goto out;
 	}
@@ -415,10 +415,10 @@ int cpbuilder_create(const struct pbuilderconfig* pc)
 out:
   if (ret)
     {
-      printf("pbuilder create failed\n");
+      log_printf(log_error, "pbuilder create failed");
       if (!pc->debug && 0!=rmrf_check_mount(pc->basepath))
 	{
-	  fprintf(stderr, "Could not remove original tree\n");
+	  log_printf(log_error, "Could not remove original tree");
 	}
     }
   return ret;
@@ -442,24 +442,24 @@ int cpbuilder_internal_saveupdate(const struct pbuilderconfig* pc)
       /* outofmemory */
       return -1;
     }
-  printf(" -> Moving work directory [%s] to final location [%s] and cleaning up old copy\n",
+  log_printf(log_info, "Moving work directory [%s] to final location [%s] and cleaning up old copy",
 	 pc->buildplace, pc->basepath);
 
   if(-1==rename(pc->basepath, buf_tmpfile))
     {
-      perror("rename");
+      log_perror("rename");
       ret=-1;
       goto out;
     }
   if(-1==rename(pc->buildplace, pc->basepath))
     {
-      perror("rename");
+      log_perror("rename");
       ret=-1;
       goto out;
     }
   if (0!=rmrf_check_mount(buf_tmpfile))
     {
-      printf("Could not remove original tree\n");
+      log_printf(log_error, "Could not remove original tree");
       ret=-1;
       goto out;
     }
@@ -476,7 +476,7 @@ int cpbuilder_login(const struct pbuilderconfig* pc)
 
   if (cpbuilder_internal_cowcopy(pc))
     {
-      fprintf(stderr, "Failed cowcopy.\n");
+      log_printf(log_error, "Failed cowcopy.");
       return 1;
     }
 
@@ -486,7 +486,7 @@ int cpbuilder_login(const struct pbuilderconfig* pc)
       /* outofmemory */
       return -1;
     }
-  printf(" -> Invoking pbuilder\n");
+  log_printf(log_info, "Invoking pbuilder");
   pbuildercommandline[1]="login";
   PBUILDER_ADD_PARAM("--buildplace");
   PBUILDER_ADD_PARAM(pc->buildplace);
@@ -511,7 +511,7 @@ int cpbuilder_login(const struct pbuilderconfig* pc)
 	}
     }
   else
-    printf("pbuilder login aborted, not cleaning\n");
+    log_printf(log_warn, "pbuilder login aborted, not cleaning");
 
   return ret;
 }
@@ -529,7 +529,7 @@ int cpbuilder_execute(const struct pbuilderconfig* pc, char** av)
 
   if (cpbuilder_internal_cowcopy(pc))
     {
-      fprintf(stderr, "Failed cowcopy.\n");
+      log_printf(log_error, "Failed cowcopy.");
       return 1;
     }
 
@@ -538,7 +538,7 @@ int cpbuilder_execute(const struct pbuilderconfig* pc, char** av)
       /* outofmemory */
       return -1;
     }
-  printf(" -> Invoking pbuilder\n");
+  log_printf(log_info, "Invoking pbuilder");
   pbuildercommandline[1]="execute";
   PBUILDER_ADD_PARAM("--buildplace");
   PBUILDER_ADD_PARAM(pc->buildplace);
@@ -570,7 +570,7 @@ int cpbuilder_execute(const struct pbuilderconfig* pc, char** av)
 	}
     }
   else
-    printf("pbuilder execute aborted, not cleaning\n");
+    log_printf(log_warn, "pbuilder execute aborted, not cleaning");
 
   return ret;
 }
@@ -629,17 +629,17 @@ int cpbuilder_update(const struct pbuilderconfig* pc)
 		 pc->buildplace))
     {
       /* if outofmemory, die. */
-      fprintf(stderr, "Out of memory.\n");
+      log_printf(log_error, "Out of memory.");
       return -1;
     }
 
   if (cpbuilder_internal_cowcopy(pc))
     {
-      fprintf(stderr, "Failed cowcopy.\n");
+      log_printf(log_error, "Failed cowcopy.");
       return 1;
     }
 
-  printf(" -> Invoking pbuilder\n");
+  log_printf(log_info, "Invoking pbuilder");
 
   pbuildercommandline[1]="update";
   PBUILDER_ADD_PARAM("--buildplace");
@@ -669,27 +669,27 @@ int cpbuilder_update(const struct pbuilderconfig* pc)
     {
       if (ret < 128)
 	{
-	  fprintf(stderr, "pbuilder update failed\n");
+	  log_printf(log_error, "pbuilder update failed");
 
 	  if (!pc->no_cowdancer_update)
 	    {
 	      /* advise the user to try this option first */
-	      fprintf(stderr, "E: could not update with cowdancer, try --no-cowdancer-update option\n");
+	      log_printf(log_error, "could not update with cowdancer, try --no-cowdancer-update option");
 	    }
 
 	  if (!pc->debug && 0!=rmrf_check_mount(pc->buildplace))
 	    {
-	      fprintf(stderr, "Could not remove original tree\n");
+	      log_printf(log_error, "Could not remove original tree");
 	    }
 	}
       else
-	printf("pbuilder update aborted, not cleaning\n");
+	log_printf(log_warn, "pbuilder update aborted, not cleaning");
 
       /* either way, I don't want to touch the original tree */
       goto out;
 
     }
-  printf(" -> removing %s working copy\n", ilist_PRGNAME);
+  log_printf(log_info, "removing %s working copy", ilist_PRGNAME);
   cpbuilder_internal_saveupdate(pc);
  out:
   free(buf_chroot);
@@ -711,7 +711,7 @@ int cpbuilder_help(void)
 	 " --basepath:\n"
 	 " --buildplace:\n"
 	 " --distribution:\n"
-	 " ... and other pbuilder options \n", ilist_PRGNAME
+	 " ... and other pbuilder options\n", ilist_PRGNAME
 	 );
   return 0;
 }
