@@ -19,6 +19,7 @@
  */
 
 #define _GNU_SOURCE
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <sys/stat.h>
@@ -114,7 +115,41 @@ int load_config_file(const char* config, pbuilderconfig* pc)
 	{
 	  /* assuming config entry */
 	  *(delim++)=0;
-	  if (!strcmp(buf, "MIRRORSITE"))
+	  if (!strcmp(buf, "LOGLEVEL"))
+	    {
+	      if (!strcmp(delim, "D"))
+		{
+		  log_set_filter_level(log_debug);
+		}
+	      else if (!strcmp(delim, "I"))
+		{
+		  log_set_filter_level(log_info);
+		}
+	      else if (!strcmp(delim, "W"))
+		{
+		  log_set_filter_level(log_warn);
+		}
+	      else if (!strcmp(delim, "E"))
+		{
+		  log_set_filter_level(log_error);
+		}
+	    }
+	  else if (!strcmp(buf, "USECOLORS"))
+	    {
+	      if (!strcmp(delim, "auto"))
+		{
+		  log_set_use_colors(log_use_colors_auto);
+		}
+	      else if (!strcmp(delim, "no"))
+		{
+		  log_set_use_colors(log_use_colors_no);
+		}
+	      else if (!strcmp(delim, "yes"))
+		{
+		  log_set_use_colors(log_use_colors_yes);
+		}
+	    }
+	  else if (!strcmp(buf, "MIRRORSITE"))
 	    {
 	      pc->mirror=strdup_strip_quote(delim);
 	    }
@@ -170,13 +205,26 @@ int load_config_file(const char* config, pbuilderconfig* pc)
 	    {
 	      pc->smp=strdup_strip_quote(delim);
 	    }
+	  else if (!strcmp(buf, "HOOKDIR"))
+	    {
+	      pc->hookdir=strdup_strip_quote(delim);
+	    }
 	  else if (!strcmp(buf, "DEBBUILDOPTS"))
 	    {
 	      pc->debbuildopts=strdup_strip_quote(delim);
 	    }
 	  else if (!strcmp(buf, "BINARY_ARCH"))
 	    {
-	      pc->binary_arch=1;
+	      if (!strcmp(delim, "binary"))
+		{
+		  pc->binary_arch=1;
+		  pc->binary_indep=0;
+		}
+	      else if (!strcmp(delim, "all"))
+		{
+		  pc->binary_arch=0;
+		  pc->binary_indep=1;
+		}
 	    }
 	  else if (!strcmp(buf, "NO_COWDANCER_UPDATE"))
 	    {
@@ -194,17 +242,29 @@ int load_config_file(const char* config, pbuilderconfig* pc)
 	    {
 	      pc->http_proxy=strdup_strip_quote(delim);
 	    }
+	  else if (!strcmp(buf, "ALLOWUNTRUSTED"))
+	    {
+	      pc->allow_untrusted=!strcmp(delim, "yes");
+	    }
+	  else if (!strcmp(buf, "BUILDRESULTUID"))
+	    {
+	      pc->buildresultuid=atoi(delim);
+	    }
+	  else if (!strcmp(buf, "BUILDRESULTGID"))
+	    {
+	      pc->buildresultgid=atoi(delim);
+	    }
 	}
     }
 
   result = WEXITSTATUS( pclose(f) );
-  if(buf) {
+  if (buf) {
     // Don't warn of missing config files
-    if( result > 1 )
-      printf( "(exit %i) -> %s\n", result, buf );
+    if (result > 1)
+      log_printf(log_error, "(exit %i) -> %s", result, buf);
     free(buf);
   }
-  if(s) free(s);
+  if (s) free(s);
   return result;
 }
 
@@ -214,16 +274,48 @@ int cpbuilder_dumpconfig(pbuilderconfig* pc)
   /* dump configuration */
   int i;
 
-  printf("dump config\n");
+  log_printf(log_info, "start dump config");
 #define DUMPINT(S) printf("  "#S": %i\n", pc->S);
 #define DUMPSTR(S) printf("  "#S": %s\n", pc->S);
-#define DUMPSTRARRAY(S) i=0; \
-  while (pc->S[i]) \
+#define DUMPSTRARRAY(S) \
+  for (i = 0; pc->S[i]; ++i) \
     { \
       printf("  "#S"[%i]: %s\n", i, pc->S[i]); \
-      i++; \
     }
 
+#define DUMPLOGLEVEL(S) printf("  log_level: "S"\n");
+#define CASELOGLEVEL(L, S) case L: DUMPLOGLEVEL(S); break;
+  switch (log_get_filter_level()) {
+    CASELOGLEVEL(log_debug, "D");
+    CASELOGLEVEL(log_info,  "I");
+    CASELOGLEVEL(log_warn,  "W");
+    CASELOGLEVEL(log_error, "E");
+    default:
+      DUMPLOGLEVEL("?");
+      break;
+  }
+
+#define DUMPUSECOLORS(S) printf("  use_colors: "S"\n")
+#define DUMPUSECOLORSAUTO(S) DUMPUSECOLORS("auto ("S")")
+#define CASEUSECOLORS(S) case log_use_colors_##S: DUMPUSECOLORS(#S); break;
+#define CASEUSECOLORSAUTO(S) case log_use_colors_##S: DUMPUSECOLORSAUTO(#S); break;
+  switch (log_get_use_colors_unresolved()) {
+    case log_use_colors_auto:
+      switch (log_get_use_colors()) {
+	CASEUSECOLORSAUTO(no);
+	CASEUSECOLORSAUTO(yes);
+	default:
+	  DUMPUSECOLORSAUTO("?");
+	  break;
+      }
+      break;
+    CASEUSECOLORS(no);
+    CASEUSECOLORS(yes);
+    default:
+      DUMPUSECOLORS("?");
+      break;
+  }
+
   DUMPINT(mountproc);
   DUMPINT(mountdev);
   DUMPINT(mountdevpts);
@@ -238,18 +330,25 @@ int cpbuilder_dumpconfig(pbuilderconfig* pc)
   DUMPSTR(components);
   DUMPSTR(othermirror);
   DUMPSTR(smp);
+  DUMPSTR(hookdir);
   DUMPSTR(debbuildopts);
   DUMPINT(binary_arch);
+  DUMPINT(binary_indep);
   DUMPSTR(http_proxy);
   DUMPSTRARRAY(inputfile);
   DUMPSTRARRAY(outputfile);
 
+  DUMPINT(buildresultuid);
+  DUMPINT(buildresultgid);
+
   DUMPINT(no_cowdancer_update);
 
   DUMPSTR(kernel_image);
   DUMPSTR(initrd);
   DUMPINT(memory_megs);
   DUMPSTR(arch);
+
+  log_printf(log_info, "end dump config");
   return 0;
 }
 
@@ -291,12 +390,15 @@ int parse_parameter(int ac, char** av,
     {"extrapackages", required_argument, 0, 0},
     {"othermirror", required_argument, 0, 0},
     {"smp", required_argument, 0, 0},
+    {"hookdir", required_argument, 0, 0},
     {"debbuildopts", required_argument, 0, 0},
     {"binary-arch", no_argument, 0, 0},
+    {"binary-indep", no_argument, 0, 0},
     {"inputfile", required_argument, 0, 0},
     {"outputfile", required_argument, 0, 0},
     {"architecture", required_argument, 0, 0},
     {"http-proxy", required_argument, 0, 0},
+    {"allow-untrusted", no_argument, 0, 0},
 
     /* cowbuilder specific options */
     {"no-cowdancer-update", no_argument, 0, 0},
@@ -318,7 +420,6 @@ int parse_parameter(int ac, char** av,
     {"debootstrap", required_argument, 0, 'M'},
 
     /* verbatim options without argument, synced as of pbuilder 0.153 */
-    {"allow-untrusted", no_argument, 0, 'm'},
     {"removepackages", no_argument, 0, 'm'},
     {"override-config", no_argument, 0, 'm'},
     {"pkgname-logfile", no_argument, 0, 'm'},
@@ -364,18 +465,21 @@ int parse_parameter(int ac, char** av,
 	    {
 	      if (mkdir(optarg, 0777)<0)
 		{
-		  perror("mkdir");
+		  log_printf(log_error,
+			     "Could not create basepath '%s': %s",
+			     optarg,
+			     strerror(errno));
 		  return 1;
 		}
 	    }
 	  else if (!pc.operation)
 	    {
-	      fprintf(stderr, "need to specify operation before --basepath option\n");
+	      log_printf(log_error, "need to specify operation before --basepath option");
 	      return 1;
 	    }
 	  if (!(pc.basepath = canonicalize_file_name(optarg)))
 	    {
-	      fprintf(stderr, "cannot canonicalize filename %s, does not exist\n", optarg);
+	      log_printf(log_error, "cannot canonicalize filename %s, does not exist", optarg);
 	      return 1;
 	    }
 	  break;
@@ -390,7 +494,7 @@ int parse_parameter(int ac, char** av,
 	  if (0>asprintf(&cmdstr, "--%s", long_options[index_point].name))
 	    {
 	      /* error */
-	      fprintf(stderr, "out of memory constructing command-line options\n");
+	      log_printf(log_error, "out of memory constructing command-line options");
 	      exit (1);
 	    }
 	  PASS_TO_PBUILDER_WITH_PARAM
@@ -399,7 +503,7 @@ int parse_parameter(int ac, char** av,
 	  if (0>asprintf(&cmdstr, "--%s", long_options[index_point].name))
 	    {
 	      /* error */
-	      fprintf(stderr, "out of memory constructing command-line options\n");
+	      log_printf(log_error, "out of memory constructing command-line options");
 	      exit (1);
 	    }
 	  PASS_TO_PBUILDER_WITH_PARAM
@@ -408,7 +512,7 @@ int parse_parameter(int ac, char** av,
 	  if (0>asprintf(&cmdstr, "--%s", long_options[index_point].name))
 	    {
 	      /* error */
-	      fprintf(stderr, "out of memory constructing command-line options\n");
+	      log_printf(log_error, "out of memory constructing command-line options");
 	      exit (1);
 	    }
 	  PBUILDER_ADD_PARAM(cmdstr);
@@ -424,7 +528,7 @@ int parse_parameter(int ac, char** av,
 	  if (0>asprintf(&cmdstr, "--%s", long_options[index_point].name))
 	    {
 	      /* error */
-	      fprintf(stderr, "out of memory constructing command-line options\n");
+	      log_printf(log_error, "out of memory constructing command-line options");
 	      exit (1);
 	    }
 
@@ -466,7 +570,7 @@ int parse_parameter(int ac, char** av,
 	      pc.inputfile[size_of_ntarray(pc.inputfile)]=strdup(optarg);
 	      if (size_of_ntarray(pc.inputfile) >= MAX_CUSTOM_FILES)
 		{
-		  fprintf(stderr, "too many inputfile options\n");
+		  log_printf(log_error, "too many inputfile options");
 		  exit (1);
 		}
 	      PASS_TO_PBUILDER_WITH_PARAM
@@ -476,7 +580,7 @@ int parse_parameter(int ac, char** av,
 	      pc.inputfile[size_of_ntarray(pc.outputfile)]=strdup(optarg);
 	      if (size_of_ntarray(pc.outputfile) >= MAX_CUSTOM_FILES)
 		{
-		  fprintf(stderr, "too many outputfile options\n");
+		  log_printf(log_error, "too many outputfile options");
 		  exit (1);
 		}
 	    }
@@ -514,6 +618,14 @@ int parse_parameter(int ac, char** av,
 	      /* this is for qemubuilder */
 	      pc.http_proxy=strdup(optarg);
 
+	      /* pass it for cowbuilder */
+	      PASS_TO_PBUILDER_WITH_PARAM
+	    }
+	  else if (!strcmp(long_options[index_point].name,"hookdir"))
+	    {
+	      /* this is for qemubuilder */
+	      pc.hookdir=strdup(optarg);
+
 	      /* pass it for cowbuilder */
 	      PASS_TO_PBUILDER_WITH_PARAM
 	    }
@@ -528,6 +640,18 @@ int parse_parameter(int ac, char** av,
 	  else if (!strcmp(long_options[index_point].name,"binary-arch"))
 	    {
 	      pc.binary_arch=1;
+	      pc.binary_indep=0;
+	      PBUILDER_ADD_PARAM(cmdstr);
+	    }
+	  else if (!strcmp(long_options[index_point].name,"binary-indep"))
+	    {
+	      pc.binary_arch=0;
+	      pc.binary_indep=1;
+	      PBUILDER_ADD_PARAM(cmdstr);
+	    }
+	  else if (!strcmp(long_options[index_point].name,"allow-untrusted"))
+	    {
+	      pc.allow_untrusted=1;
 	      PBUILDER_ADD_PARAM(cmdstr);
 	    }
 	  break;
@@ -536,14 +660,14 @@ int parse_parameter(int ac, char** av,
 	  pc.operation=pbuilder_help;
 	  break;
 	default:
-	  fprintf(stderr, "Unhandled option\n");
+	  log_printf(log_error, "Unhandled option");
 	  /* Error case. */
 	  return 1;
 	}
     }
 
   if( 0 != config_ok ) {
-    printf( "Couldn't load any valid config file.\n" );
+    log_printf(log_error, "Couldn't load any valid config file.");
     exit( 6 );
   }
 
@@ -552,7 +676,7 @@ int parse_parameter(int ac, char** av,
     asprintf(&(pc.basepath), "/var/cache/pbuilder/base.%s", keyword);
   if (!pc.buildplace)
     {
-      fprintf(stderr, "E: BUILDPLACE is not set\n");
+      log_printf(log_error, "BUILDPLACE is not set");
       return 1;
     }
   else
@@ -576,8 +700,22 @@ int parse_parameter(int ac, char** av,
   if (!pc.distribution)
     pc.distribution=strdup("sid");
 
+  if (!pc.buildresultuid)
+    {
+      const char* sudo_uid = getenv("SUDO_UID");
+      if (sudo_uid)
+	pc.buildresultuid = atoi(sudo_uid);
+    }
+
+  if (!pc.buildresultgid)
+    {
+      const char* sudo_gid = getenv("SUDO_GID");
+      if (sudo_gid)
+	pc.buildresultgid = atoi(sudo_gid);
+    }
+
   if (!pc.memory_megs)
-    pc.memory_megs=128;
+    pc.memory_megs=256;
 
   switch(pc.operation)
     {
@@ -585,7 +723,7 @@ int parse_parameter(int ac, char** av,
       if (!av[optind])
 	{
 	  /* parameter missing */
-	  fprintf(stderr, "E: parameter missing for build operation\n");
+	  log_printf(log_error, "parameter missing for build operation");
 	  return 1;
 	}
 
@@ -595,7 +733,7 @@ int parse_parameter(int ac, char** av,
       if (av[optind])
 	{
 	  /* extra parameter */
-	  fprintf(stderr, "E: too many parameters for create\n");
+	  log_printf(log_error, "too many parameters for create");
 	  return 1;
 	}
       return cpbuilder_create(&pc);
@@ -604,7 +742,7 @@ int parse_parameter(int ac, char** av,
       if (av[optind])
 	{
 	  /* extra parameter */
-	  fprintf(stderr, "E: too many parameters for update\n");
+	  log_printf(log_error, "too many parameters for update");
 	  return 1;
 	}
       return cpbuilder_update(&pc);
@@ -616,7 +754,7 @@ int parse_parameter(int ac, char** av,
       if (!av[optind])
 	{
 	  /* parameter missing */
-	  fprintf(stderr, "E: parameter missing for execute operation\n");
+	  log_printf(log_error, "parameter missing for execute operation");
 	  return 1;
 	}
       return cpbuilder_execute(&pc, &av[optind]);
@@ -628,7 +766,7 @@ int parse_parameter(int ac, char** av,
       return cpbuilder_dumpconfig(&pc);
 
     default:
-      fprintf (stderr, "E: No operation specified\n");
+      log_printf(log_error, "No operation specified");
       return 1;
     }
 
