@@ -519,10 +519,12 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
       /* this is the child process */
       const char* qemu = qemu_arch_qemu(pc->arch);
       const char* machine = qemu_arch_qemumachine(pc->arch);
+      const char* cpu = qemu_arch_cpu(pc->arch);
       const char* hda_format = format_for_image(hda);
       const char* hdb_format = format_for_image(hdb);
       char* hda_command;
       char* hdb_command;
+      int virt;
       char* append_command;
       const char* kernel_image = pc->kernel_image;
       const char* initrd = pc->initrd;
@@ -566,12 +568,16 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
 	  exit(1);
 	}
 
+      virt = !strcmp(machine, "virt");
+
       asprintf(&hda_command,
-	       "file=%s,format=%s,index=0,media=disk,cache=writeback,id=hd0",
+	       "%sfile=%s,format=%s,index=0,media=disk,cache=writeback,id=hd0",
+	       virt ? "if=none," : "",
 	       hda, hda_format);
 
       asprintf(&hdb_command,
-	       "file=%s,format=%s,index=1,media=disk,cache=writeback,id=hd1",
+	       "%sfile=%s,format=%s,index=1,media=disk,cache=writeback,id=hd1",
+	       virt ? "if=none," : "",
 	       hdb, hdb_format);
 
       /* panic < 0 means reboot immediately on panic; this will actually
@@ -592,6 +598,11 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
       argv[argc++]="-no-reboot"; /* halt instead of rebooting */
       argv[argc++]="-M";
       argv[argc++]=strdupa(machine);
+      if (cpu)
+	{
+	  argv[argc++]="-cpu";
+	  argv[argc++]=strdupa(cpu);
+	}
       argv[argc++]="-m";
       argv[argc++]=mem;
       if (pc->smp) {
@@ -622,21 +633,24 @@ static int fork_qemu(const char* hda, const char* hdb, const struct pbuilderconf
 	}
       argv[argc++]="-net";
       argv[argc++]="user";
-      if (!strcmp(machine, "virt")) {
-	argv[argc++]="-device";
-	argv[argc++]="virtio-scsi-device,id=scsi";
-	argv[argc++]="-device";
-	argv[argc++]="virtio-net-device,netdev=net0";
-	argv[argc++]="-device";
-	argv[argc++]="scsi-hd,drive=hd0";
-	argv[argc++]="-device";
-	argv[argc++]="scsi-hd,drive=hd1";
-	argv[argc++]="-netdev";
-	argv[argc++]="user,id=net0";
-      } else {
-	argv[argc++]="-net";
-	argv[argc++]="nic";
-      }
+      if (virt)
+	{
+	  argv[argc++]="-device";
+	  argv[argc++]="virtio-scsi-device,id=scsi";
+	  argv[argc++]="-device";
+	  argv[argc++]="virtio-net-device,netdev=net0";
+	  argv[argc++]="-device";
+	  argv[argc++]="scsi-hd,drive=hd0";
+	  argv[argc++]="-device";
+	  argv[argc++]="scsi-hd,drive=hd1";
+	  argv[argc++]="-netdev";
+	  argv[argc++]="user,id=net0";
+	}
+      else
+	{
+	  argv[argc++]="-net";
+	  argv[argc++]="nic";
+	}
       argv[argc]=NULL;
       assert(argc < MAX_ARGS);
 
